#!/usr/bin/env python

# GNU GENERAL PUBLIC LICENSE (GPLv3)
# Copyright (c) 2020 - 2021 Bernardo Bulgarelli Labronici
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>5.



# Error Level Analisys in GIMP as described by
#  Neal Krawetz and Hacker Factor Solutions. A pictures
# worth... Hacker Factor Solutions, 6, 2007.


# Feedback welcome: bernardobl@gmail.com



import os
import sys
#Current folder is not with sys path. cannot load GFUtils
sys.path.append(os.path.dirname(__file__))

from gimpfu import *
from array import array
from GFutils import image_info


def ELA(filepath,quality,gamma,info,batch,step):

    #pdb.gimp_message()

    #open image
    opImage = pdb.file_jpeg_load(filepath,filepath)

    if batch:
        qual = quality
        while qual < 1.00:
            DO_ELA(opImage,filepath,qual,gamma,info)
            qual += step
    else:
        DO_ELA(opImage,filepath,quality,gamma,info)

def DO_ELA(opImage,filepath,quality,gamma,info):
    
    layer = opImage.layers[0]
    
    #new image with two layer
    img = pdb.gimp_image_new(opImage.width, opImage.height, RGB)
    newLayer = img.new_layer("",pos=1,fill_mode=WHITE_FILL)
    #first layer will make the difference with second one
    newLayer.mode = LAYER_MODE_DIFFERENCE
    newLayerII = img.new_layer("",pos=1,fill_mode=WHITE_FILL)
    
    #copy original image to first layer
    pdb.gimp_edit_copy (layer)
    floating = pdb.gimp_edit_paste (newLayer,0)
    pdb.gimp_floating_sel_anchor(floating)
    
    #save lower quality
    temp = "ELAtemp" + str(quality) + ".jpg"
    #pdb.gimp_message("quality : " + str(quality) + " - " +temp)
    pdb.file_jpeg_save(opImage,layer,temp,temp,quality,0,0,0,'none',0,0,0,0)
    
    #copy lower quality image to second layer
    lowerimg = pdb.file_jpeg_load(temp,temp)
    lowerLayer = lowerimg.layers[0]
    pdb.gimp_edit_copy (lowerLayer)
    floating = pdb.gimp_edit_paste (newLayerII,0)
    pdb.gimp_floating_sel_anchor(floating)
    
    #merge layer
    finalLayer = pdb.gimp_image_merge_visible_layers(img, 0)
    
    #get ll pixels to calculate maximum value present to final image to ajust levels of brightness/contrast accordingly
    region = finalLayer.get_pixel_rgn(0,0,finalLayer.width,finalLayer.height,False,False)
    #copy array for performance (up to 10x faster)
    pixels = array("B", region[0:finalLayer.width,0:finalLayer.height])
      
    
    #Now the Histogram value will be adjust. First the lower and maximum values must be computed
    maxV = 0
    minV = 255   

    if region.bpp < 3: #Monocromatic Image (only value channel)
        for x in range(finalLayer.width * finalLayer.height):
          cr = pixels[x]
          if cr > maxV:
            maxV = cr
          if cr < minV:
            minV = cr     
    else: #RGB image
        for x in range(finalLayer.width * finalLayer.height):
          pos = x * region.bpp  
          cr = max(pixels[pos], pixels[pos + 1], pixels[pos + 2])
          if cr > maxV:
            maxV = cr
          if cr < minV:
            minV = cr    
    
    
    os.remove(temp)
    
    #preset gamma value
    pdb.gimp_drawable_levels(finalLayer, HISTOGRAM_VALUE, minV/255.0, maxV/255.0, 0, gamma, 0, 1.0, 0)
     
    if info: 
        image_info(img,"quality: " + str(quality*100) + "%\ngamma: " + str(gamma)) 
    
    display=pdb.gimp_display_new(img)
    


register(
    "python-fu-ela",
    "Error level analysis",
    "Performe an Error Level Analysis in the selected image. Abnormal compression level is highlighted",
    "Bernardo Bulgarelli Labronici", "Bernardo Bulgarelli Labronici", "2020",
    "Error level analysis",
    "", 
    [
        (PF_FILE, "filepath", "File to analyze", "/Default/Path"),
        (PF_FLOAT, "quality", "Jpeg quality", 0.9),
        (PF_FLOAT, "gamma", "Gamma for level adjustment", 1.0),
        (PF_BOOL, "info","Attach info to image",True),
        (PF_BOOL, "batch","Batch Render",False),
        (PF_FLOAT, "step","Step for Batch Render", 0.05)
        
    ],
    [],
    ELA, menu="<Image>/Forensics")

main()