package com.jakub.labchessboard;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.os.Bundle;
import android.view.Gravity;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.Window;
import android.widget.Toast;

public class MainActivity extends AppCompatActivity {

    private Panel mPanel;
    private final Paint backgroundFirstField = new Paint();
    private final Paint backgroundSecondField = new Paint();

    private float widthPanel;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        requestWindowFeature(Window.FEATURE_NO_TITLE);

        mPanel = new Panel(this);
        setContentView(mPanel);
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.main, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(@NonNull MenuItem item) {
        switch (item.getItemId()) {
            case R.id.menu_item_creator:
                Toast toast = Toast.makeText(this, "Moje Imię i Nazwisko", Toast.LENGTH_SHORT);
                toast.setGravity(Gravity.BOTTOM|Gravity.BOTTOM, 0, 0);
                toast.show();
                return true;
            case R.id.menu_item_exit:
                finish();
                return true;
            case R.id.menu_item_color_whiteblack:
                backgroundFirstField.setColor(Color.WHITE);
                backgroundSecondField.setColor(Color.BLACK);
                mPanel.invalidate();
                return true;
            case R.id.menu_item_color_redyellow:
                backgroundFirstField.setColor(Color.RED);
                backgroundSecondField.setColor(Color.YELLOW);
                mPanel.invalidate();
                return true;
            default:
                return super.onOptionsItemSelected(item);
        }
    }

    public class Panel extends View {

        public Panel(Context context) {
            super(context);
            backgroundFirstField.setColor(Color.WHITE);
            backgroundSecondField.setColor(Color.BLACK);
        }

        @Override
        protected void onDraw(Canvas canvas) {
            canvas.drawColor(Color.GRAY);

            float width = (widthPanel == 0) ? mPanel.getWidth() : Math.min(widthPanel, mPanel.getWidth());
            widthPanel = mPanel.getWidth();

            for (int i = 0; i < 8; i++)
                for (int j = 0; j < 8; j++)
                    if ((i + j) % 2 == 0) {
                        canvas.drawRect(i * width/8, j * width/8, (i + 1)*width/8, (j + 1)*width/8, backgroundFirstField);
                    }
                    else {
                        canvas.drawRect(i * width/8, j * width/8, (i + 1)*width/8, (j + 1)*width/8, backgroundSecondField);
                    }
        }
    }
}