package com.jakub.labintencje;

import androidx.activity.result.ActivityResultLauncher;
import androidx.activity.result.contract.ActivityResultContracts;
import androidx.annotation.NonNull;
import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.app.ActivityCompat;

import android.Manifest;
import android.content.Context;
import android.content.DialogInterface;
import android.content.pm.PackageManager;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;

import java.util.Locale;

public class GeoActivity extends AppCompatActivity implements LocationListener {

    private Button mGeoStartSearch;
    private Button mGeoStopSearch;
    private TextView mGeoLatitude;
    private TextView mGeoLongitude;

    private LocationManager mLocationManager;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_geo);

        mGeoStartSearch = findViewById(R.id.btn_start_geo_search);
        mGeoStopSearch = findViewById(R.id.btn_stop_geo_search);
        mGeoLatitude = findViewById(R.id.txt_geo_latitude);
        mGeoLongitude = findViewById(R.id.txt_geo_longitude);

        mGeoStartSearch.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                startSearchLocation();
            }
        });

        mGeoStopSearch.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                stopSearchLocation();
            }
        });

        mLocationManager = (LocationManager) getSystemService(Context.LOCATION_SERVICE);
    }

    private void exitActivity() {
        finish();
    }

    private void startSearchLocation() {
        if (ActivityCompat.checkSelfPermission(this,
                Manifest.permission.ACCESS_COARSE_LOCATION) != PackageManager.PERMISSION_GRANTED &&
                ActivityCompat.checkSelfPermission(this, Manifest.permission.ACCESS_COARSE_LOCATION) != PackageManager.PERMISSION_GRANTED) {

            requestCourseLocationPermissionLauncher.launch(Manifest.permission.ACCESS_COARSE_LOCATION);

            return;
        }
        if (ActivityCompat.checkSelfPermission(this,
                Manifest.permission.ACCESS_FINE_LOCATION) != PackageManager.PERMISSION_GRANTED &&
                ActivityCompat.checkSelfPermission(this, Manifest.permission.ACCESS_FINE_LOCATION) != PackageManager.PERMISSION_GRANTED) {

            requestCourseLocationPermissionLauncher.launch(Manifest.permission.ACCESS_FINE_LOCATION);

            return;
        }

        mLocationManager.requestLocationUpdates(
                LocationManager.GPS_PROVIDER, 0, 0, this);
        enabledStartSearch(false);
        enabledStopSearch(true);
    }

    private void stopSearchLocation() {
        mLocationManager.removeUpdates(this);
        enabledStopSearch(false);
        enabledStartSearch(true);
    }

    private void enabledStartSearch(boolean enable) {
        mGeoStartSearch.setEnabled(enable);
    }

    private void enabledStopSearch(boolean enable) {
        mGeoStopSearch.setEnabled(enable);
    }

    private void setLongitude(double value) {
        mGeoLongitude.setText(String.format(Locale.ENGLISH,"%.6f", value));
    }

    private void setLatitude(double value) {
        mGeoLatitude.setText(String.format(Locale.ENGLISH,"%.6f", value));
    }

    @Override
    public void onLocationChanged(@NonNull Location location) {
        setLatitude(location.getLatitude());
        setLongitude(location.getLongitude());
    }

    @Override
    public void onStatusChanged(String provider, int status, Bundle extras) {

    }

    @Override
    public void onProviderEnabled(@NonNull String provider) {

    }

    @Override
    public void onProviderDisabled(@NonNull String provider) {

    }

    @Override
    public void onBackPressed() {
        stopSearchLocation();
    }

    private final ActivityResultLauncher<String> requestCourseLocationPermissionLauncher =
            registerForActivityResult(new ActivityResultContracts.RequestPermission(), isGranted -> {
                if (isGranted) {
                    startSearchLocation();
                } else {
                    AlertDialog.Builder builder = new AlertDialog.Builder(getParent());
                    builder.setTitle(getString(R.string.dialog_access_gps_warning));
                    builder.setMessage(getString(R.string.dialog_access_gps_question));
                    builder.setPositiveButton(getString(R.string.dialog_button_ok), new DialogInterface.OnClickListener() {
                        public void onClick(DialogInterface dialog, int id) {
                            startSearchLocation();
                        }
                    });
                    builder.setNegativeButton(getString(R.string.dialog_button_cancel), new DialogInterface.OnClickListener() {
                        public void onClick(DialogInterface dialog, int id) {
                            exitActivity();
                        }
                    });
                    AlertDialog dialog = builder.create();
                    dialog.show();
                }
            });
}